﻿; Database procedures

Declare OpenDB()
Declare CloseDB()
Declare QueryContacts(SQLCmd.s)
Declare GetContacts()
Declare InsertContacts()
Declare UpdateContacts()
Declare DeleteContacts()
Declare DB2Record()
Declare List2Record()
Declare SetBoundVars()

Structure ContactsStructure
  ID.l
  FullName.s
  Phone.s
  Email.s
  WebURL.s
EndStructure

Global ContactsRecord.ContactsStructure
Global NewList ContactsList.ContactsStructure()

Enumeration
  #DBContacts_ID
  #DBContacts_FullName
  #DBContacts_Phone
  #DBContacts_Email
  #DBContacts_WebURL
EndEnumeration

Dim ContactsCols.s(4)
ContactsCols(#DBContacts_ID) = "ID"
ContactsCols(#DBContacts_FullName) = "FullName"
ContactsCols(#DBContacts_Phone) = "Phone"
ContactsCols(#DBContacts_Email) = "Email"
ContactsCols(#DBContacts_WebURL) = "WebURL"

Procedure OpenDB()
  AppData\Database = OpenDatabase(#PB_Any, AppData\DBName, "", "")
  If AppData\Database = 0
    MessageRequester("Error", DatabaseError(), #PB_MessageRequester_Ok | #PB_MessageRequester_Error)
    End
  EndIf
EndProcedure

Procedure CloseDB()
  CloseDatabase(AppData\Database)
EndProcedure

Procedure QueryContacts(sql.s)
  OpenDB()
  ClearList(ContactsList())
  If DatabaseQuery(AppData\Database, sql) = 0
    MessageRequester("Error", DatabaseError(), #PB_MessageRequester_Ok | #PB_MessageRequester_Error)
    End
  Else
    While NextDatabaseRow(AppData\Database)
      AddElement(ContactsList())
      DB2Record()
      ContactsList() = ContactsRecord
    Wend
  EndIf
  CloseDB()
EndProcedure

Procedure GetContacts()
  OpenDB()
  SetDatabaseLong(AppData\Database, 0, ContactsRecord\ID)
  If DatabaseQuery(AppData\Database, "SELECT * FROM Contacts WHERE id = ?;") = 0
    MessageRequester("Error", DatabaseError(), #PB_MessageRequester_Ok | #PB_MessageRequester_Error)
    End
  Else
    NextDatabaseRow(AppData\Database)
    DB2Record()
  EndIf
  CloseDB()
EndProcedure

Procedure InsertContacts()
  Shared ContactsCols()
  Define sql.s
  OpenDB()
  SetBoundVars()
  sql = "INSERT INTO Contacts (" +
        ContactsCols(#DBContacts_FullName) + ", " +
        ContactsCols(#DBContacts_Phone) + ", " +
        ContactsCols(#DBContacts_Email) + ", " +
        ContactsCols(#DBContacts_WebURL) + ") VALUES (?, ?, ?, ?);"
  If DatabaseUpdate(AppData\Database, sql) = 0
    MessageRequester("Error", DatabaseError(), #PB_MessageRequester_Ok | #PB_MessageRequester_Error)
    ContactsRecord\ID = 0
  Else
    DatabaseQuery(AppData\Database, "SELECT @@IDENTITY;")
    NextDatabaseRow(AppData\Database)
    ContactsRecord\ID = GetDatabaseLong(AppData\Database, 0)
    LastElement(ContactsList())
    AddElement(ContactsList())
    ContactsList() = ContactsRecord
  EndIf
  CloseDB()
EndProcedure

Procedure UpdateContacts()
  Shared ContactsCols()
  Define sql.s
  OpenDB()
  SetBoundVars()
  SetDatabaseLong(AppData\Database, 4, ContactsRecord\ID)
  sql = "UPDATE Contacts SET " +
        ContactsCols(#DBContacts_FullName) + " = ?, "  +
        ContactsCols(#DBContacts_Phone) + " = ?, "  +
        ContactsCols(#DBContacts_Email) + " = ?, "  +
        ContactsCols(#DBContacts_WebURL) + " = ? "  +
        "WHERE " + ContactsCols(#DBContacts_ID) + " = ?;"
  If DatabaseUpdate(AppData\Database, sql) = 0
    MessageRequester("Error", DatabaseError(), #PB_MessageRequester_Ok | #PB_MessageRequester_Error)
    ContactsRecord\ID = 0
  Else
    ContactsList() = ContactsRecord
  EndIf
  CloseDB()
EndProcedure

Procedure DeleteContacts()
  Define sql.s
  OpenDB()
  SetDatabaseLong(AppData\Database, 0, ContactsRecord\ID)
  sql = "DELETE FROM Contacts WHERE id = ?;"
  If DatabaseUpdate(AppData\Database, sql) = 0
    MessageRequester("Error", DatabaseError(), #PB_MessageRequester_Ok | #PB_MessageRequester_Error)
    End
  EndIf
  DeleteElement(ContactsList(), 1)
  If ListSize(ContactsList()) = 0
    ContactsRecord\ID = 0
  Else
    List2Record()
  EndIf
  CloseDB()
EndProcedure

Procedure DB2Record()
  ContactsRecord\ID = GetDatabaseLong(AppData\Database, #DBContacts_ID)
  ContactsRecord\FullName = GetDatabaseString(AppData\Database, #DBContacts_FullName)
  ContactsRecord\Phone = GetDatabaseString(AppData\Database, #DBContacts_Phone)
  ContactsRecord\Email = GetDatabaseString(AppData\Database, #DBContacts_Email)
  ContactsRecord\WebURL = GetDatabaseString(AppData\Database, #DBContacts_WebURL)
EndProcedure

Procedure List2Record()
  ContactsRecord\ID = ContactsList()\ID
  ContactsRecord\FullName = ContactsList()\FullName
  ContactsRecord\Phone = ContactsList()\Phone
  ContactsRecord\Email = ContactsList()\Email
  ContactsRecord\WebURL = ContactsList()\WebURL
EndProcedure

Macro BindStringVar(Ndx, Var)
  If ContactsRecord\Var = ""
    SetDatabaseNull(AppData\Database, Ndx)
  Else
    SetDatabaseString(AppData\Database, Ndx, ContactsRecord\Var)
  EndIf
EndMacro

Procedure SetBoundVars()
  BindStringVar(0, FullName)
  BindStringVar(1, Phone)
  BindStringVar(2, Email)
  BindStringVar(3, WebURL)
EndProcedure
